<?php

namespace Drupal\sendgrid_integration_simplenews_reports\Controller;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Link;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Url;
use Drupal\node\NodeInterface;
use Drupal\sendgrid_integration_simplenews_reports\Form\DateRangeForm;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\sendgrid_integration_reports\Api;
use Drupal\sendgrid_integration_simplenews_reports\Service\SendGridSimplenewsReportsService;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpFoundation\Response;

/**
 * Displays SendGrid email statistics for Simplenews newsletters.
 */
class SendGridSimplenewsReportsController extends ControllerBase {

  /**
   * The SendGrid API service.
   *
   * @var \Drupal\sendgrid_integration_reports\Api
   */
  protected $api;

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The SendGrid Simplenews Reports Service.
   *
   * @var \Drupal\sendgrid_integration_simplenews_reports\Service\SendGridSimplenewsReportsService
   */
  protected $reportsService;

  /**
   * Constructs a SendGridSimplenewsReportsController object.
   *
   * @param \Drupal\sendgrid_integration_reports\Api $api
   *   The SendGrid API service.
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack.
   * @param \Drupal\sendgrid_integration_simplenews_reports\Service\SendGridSimplenewsReportsService $reports_service
   *   The SendGrid Simplenews Reports Service.
   */
  public function __construct(
    Api $api,
    RequestStack $request_stack,
    SendGridSimplenewsReportsService $reports_service
  ) {
    $this->api = $api;
    $this->requestStack = $request_stack;
    $this->reportsService = $reports_service;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('sendgrid_integration_reports.api'),
      $container->get('request_stack'),
      $container->get('sendgrid_integration_simplenews_reports.reports_service')
    );
  }

  /**
   * Checks if the node is a Simplenews newsletter.
   *
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The user account.
   * @param \Drupal\node\NodeInterface $node
   *   The node to check.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   The access result.
   */
  public function checkNodeAccess(AccountInterface $account, NodeInterface $node) {
    return $node->hasField('simplenews_issue')
      ? AccessResult::allowed()
      : AccessResult::forbidden();
  }

  /**
   * Title callback for the node reports page.
   *
   * @param \Drupal\node\NodeInterface $node
   *   The newsletter node.
   *
   * @return \Drupal\Core\StringTranslation\TranslatableMarkup
   *   The page title.
   */
  public function getNodeReportsTitle(NodeInterface $node) {
    $date_range = $this->reportsService->getDateRangeFromRequest($node);

    return $this->t('Statistics for "@title" (@start_date to @end_date)', [
      '@title' => $node->getTitle(),
      '@start_date' => $date_range['start_date'],
      '@end_date' => $date_range['end_date'],
    ]);
  }

  /**
   * Displays email reports for a newsletter node or exports as CSV.
   *
   * @param \Drupal\node\NodeInterface $node
   *   The newsletter node.
   *
   * @return array|Response
   *   Render array for the report page or CSV response.
   */
  public function getNodeReports(NodeInterface $node) {
    $date_range = $this->reportsService->getDateRangeFromRequest($node);
    $start_date = $date_range['start_date'];
    $end_date = $date_range['end_date'];

    // Get email statistics data from the service.
    $statistics = $this->reportsService->getNewsletterStatistics(
      $node->id(),
      $start_date,
      $end_date
    );

    // Check if CSV export is requested
    $request = $this->requestStack->getCurrentRequest();
    if ($request->query->get('format') === 'csv') {
      return $this->exportAsCsv($statistics, $node);
    }

    // Get the date range form.
    $date_range_form = $this->formBuilder()->getForm(
      DateRangeForm::class,
      $node,
      $start_date,
      $end_date
    );

    // Build report render array.
    $render = $this->buildReportPage(
      $statistics,
      $node->getTitle(),
      $start_date,
      $end_date
    );

    // Add date range form at the top.
    $render['date_range_form'] = [
      '#type' => 'container',
      '#weight' => -100,
      'form' => $date_range_form,
    ];

    // Add CSV export link.
    $csv_url = Url::fromRoute('sendgrid_integration_simplenews_reports.node_statistics', [
      'node' => $node->id(),
    ], [
      'query' => [
        'start_date' => $start_date,
        'end_date' => $end_date,
        'format' => 'csv',
      ],
    ]);

    $render['export_links'] = [
      '#type' => 'container',
      '#weight' => 100,
      '#attributes' => ['class' => ['sendgrid-export-links']],
      'csv' => [
        '#type' => 'link',
        '#title' => $this->t('Export as CSV'),
        '#url' => $csv_url,
        '#attributes' => [
          'class' => ['button', 'button--small'],
        ],
      ],
    ];

    $render['#prefix'] = '<div class="sendgrid-integration-simplenews-reports-container">';
    $render['#suffix'] = '</div>';
    return $render;
  }

  /**
   * Exports statistics as CSV.
   *
   * @param array $statistics
   *   The statistics data.
   * @param \Drupal\node\NodeInterface $node
   *   The newsletter node.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   CSV response.
   */
  protected function exportAsCsv(array $statistics, NodeInterface $node) {
    // Generate CSV content.
    $csv_content = $this->generateCsvContent($statistics, $node->getTitle());

    // Create response with CSV content.
    $response = new Response($csv_content);
    $response->headers->set('Content-Type', 'text/csv; charset=utf-8');
    $response->headers->set('Content-Disposition', 'attachment; filename="sendgrid_statistics_' . $node->id() . '_' . date('Y-m-d') . '.csv"');

    return $response;
  }

  /**
   * Generates CSV content from statistics data.
   *
   * @param array $stats
   *   Statistics data from SendGrid.
   * @param string $newsletter_title
   *   The newsletter title.
   *
   * @return string
   *   CSV formatted content.
   */
  protected function generateCsvContent(array $stats, string $newsletter_title) {
    $metrics = $this->reportsService->getMetricDefinitions();
    $csv_rows = [];

    // Add header row.
    $header = ['Date'];
    foreach ($metrics as $metric_key => $metric_label) {
      $header[] = (string) $metric_label;
    }
    $csv_rows[] = $header;

    // Add data rows.
    if (!empty($stats['global'])) {
      foreach ($stats['global'] as $data_point) {
        $row = [$data_point['date']];
        foreach (array_keys($metrics) as $metric) {
          $row[] = isset($data_point[$metric]) ? $data_point[$metric] : 0;
        }
        $csv_rows[] = $row;
      }

      // Add totals row.
      $totals = array_fill_keys(array_keys($metrics), 0);
      foreach ($stats['global'] as $item) {
        foreach ($totals as $metric => $count) {
          if (isset($item[$metric])) {
            $totals[$metric] += $item[$metric];
          }
        }
      }

      $total_row = ['Total'];
      foreach (array_keys($metrics) as $metric) {
        $total_row[] = $totals[$metric];
      }
      $csv_rows[] = $total_row;
    }

    // Convert array to CSV.
    $csv_content = '';
    foreach ($csv_rows as $row) {
      $csv_content .= $this->csvEscapeRow($row) . "\n";
    }

    return $csv_content;
  }

  /**
   * Escapes and formats a row for CSV output.
   *
   * @param array $row
   *   Array of values to format as CSV.
   *
   * @return string
   *   CSV formatted row.
   */
  protected function csvEscapeRow(array $row) {
    $escaped_row = [];
    foreach ($row as $value) {
      // Escape double quotes and wrap in quotes if contains comma, quote or newline.
      $value = str_replace('"', '""', $value);
      if (preg_match('/[,"\n\r]/', $value)) {
        $value = '"' . $value . '"';
      }
      $escaped_row[] = $value;
    }
    return implode(',', $escaped_row);
  }

  /**
   * Builds the report page render array.
   *
   * @param array $stats
   *   Statistics data from SendGrid.
   * @param string $newsletter_title
   *   The newsletter title.
   * @param string $start_date
   *   Start date in YYYY-MM-DD format.
   * @param string $end_date
   *   End date in YYYY-MM-DD format.
   *
   * @return array
   *   Render array for the report page.
   */
  protected function buildReportPage(
    array $stats,
    string $newsletter_title,
    string $start_date,
    string $end_date
  ) {
    $chart_data = $this->prepareChartData($stats);

    $render = [
      '#attached' => [
        'library' => [
          'sendgrid_integration_reports/googlejsapi',
          'sendgrid_integration_reports/main',
        ],
        'drupalSettings' => [
          'sendgrid_integration_reports' => $chart_data,
        ],
      ],
      '#attributes' => ['class' => ['sendgrid-integration-simplenews-reports-container']],
    ];

    // Add charts.
    $render['volume'] = [
      '#prefix' => '<h2>' . $this->t('Sending Volume') . '</h2>',
      '#markup' => '<div id="sendgrid-global-volume-chart"></div>',
    ];

    $render['spam'] = [
      '#prefix' => '<h2>' . $this->t('Spam Reports') . '</h2>',
      '#markup' => '<div id="sendgrid-global-spam-chart"></div>',
    ];

    // Add statistics summary table if we have global stats.
    if (!empty($stats['global'])) {
      $render['statistics_summary'] = [
        '#type' => 'container',
        'title' => [
          '#markup' => '<h2>' . $this->t('Statistics Summary') . '</h2>',
        ],
        'table' => $this->buildStatisticsSummary($stats['global']),
      ];
    }

    // Add explanatory message.
    $render['message'] = [
      '#markup' => $this->t(
        'The following reports are for the newsletter: %title from %start_date to %end_date. For more comprehensive data, please visit your @dashboard.',
        [
          '%title' => $newsletter_title,
          '%start_date' => $start_date,
          '%end_date' => $end_date,
          '@dashboard' => Link::fromTextAndUrl(
            $this->t('SendGrid Dashboard'),
            Url::fromUri('//app.sendgrid.com/')
          )->toString(),
        ]
      ),
    ];

    return $render;
  }

  /**
   * Prepares chart data for JavaScript.
   *
   * @param array $stats
   *   The SendGrid statistics data.
   *
   * @return array
   *   Formatted data for charts.
   */
  protected function prepareChartData(array $stats) {
    $chart_data = [];
    if (!empty($stats['global'])) {
      foreach ($stats['global'] as $data_point) {
        $chart_data['global'][] = [
          'date' => $data_point['date'],
          'opens' => $data_point['opens'],
          'clicks' => $data_point['clicks'],
          'delivered' => $data_point['delivered'],
          'spam_reports' => $data_point['spam_reports'],
          'spam_report_drops' => $data_point['spam_report_drops'],
        ];
      }
    }
    return $chart_data;
  }

  /**
   * Builds a summary table of email statistics.
   *
   * @param array $data
   *   The statistics data from SendGrid.
   *
   * @return array
   *   Render array for the summary table.
   */
  protected function buildStatisticsSummary(array $data) {
    $metrics = $this->reportsService->getMetricDefinitions();
    $totals = array_fill_keys(array_keys($metrics), 0);
    foreach ($data as $item) {
      foreach ($totals as $metric => $count) {
        if (isset($item[$metric])) {
          $totals[$metric] += $item[$metric];
        }
      }
    }
    $rows = [];
    foreach ($totals as $metric => $count) {
      $rows[] = [
        $metrics[$metric],
        $count,
      ];
    }
    return [
      '#theme' => 'table',
      '#rows' => $rows,
      '#attributes' => ['class' => ['statistics-summary-table']],
      '#header' => [
        $this->t('Metric'),
        $this->t('Count'),
      ],
    ];
  }



}
