<?php

namespace Drupal\sendgrid_integration_simplenews_reports\Service;

use Drupal\sendgrid_integration_reports\Api;
use Drupal\shs\StringTranslationTrait;
use Symfony\Component\HttpFoundation\RequestStack;
use Drupal\node\NodeInterface;

/**
 * Service for SendGrid Simplenews Reports data handling.
 *
 * @service sendgrid_integration_simplenews_reports.reports_service
 */
class SendGridSimplenewsReportsService {

  use StringTranslationTrait;

  /**
   * Constructs the SendGridSimplenewsReportsService.
   *
   * @param \Drupal\sendgrid_integration_reports\Api $api
   *   The SendGrid API service. (service: sendgrid_integration_reports.api)
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   The request stack. (service: request_stack)
   */
  public function __construct(
    private readonly Api $api,
    private readonly RequestStack $requestStack,
  ) {}

  /**
   * Gets statistics data from SendGrid API for a newsletter.
   *
   * @param int $nodeId
   *   The newsletter node ID.
   * @param string $startDate
   *   Start date (YYYY-MM-DD).
   * @param string $endDate
   *   End date (YYYY-MM-DD).
   *
   * @return array
   *   SendGrid statistics data.
   */
  public function getNewsletterStatistics($nodeId, $startDate, $endDate) {
    $categories = ['node_' . $nodeId];
    return $this->api->getStats(
      'simplenews_sendgrid_stat_' . $nodeId,
      $categories,
      $startDate,
      $endDate,
      TRUE
    );
  }

  /**
   * Gets date range from the current request with defaults.
   *
   * @param \Drupal\node\NodeInterface|null $node
   *   The node to use for the default start date.
   *
   * @return array
   *   Array with start_date and end_date.
   */
  public function getDateRangeFromRequest(?NodeInterface $node = NULL) {
    $request = $this->requestStack->getCurrentRequest();

    $defaultStartDate = ($node && $node->created->value)
      ? date('Y-m-d', $node->created->value)
      : date('Y-m-d', strtotime('-1 month'));

    return [
      'start_date' => $request->query->get('start_date', $defaultStartDate),
      'end_date' => $request->query->get('end_date', date('Y-m-d')),
    ];
  }

  /**
   * Returns definitions of email metrics with human-readable labels.
   *
   * @return array
   *   Array of metric labels keyed by metric name.
   */
  public function getMetricDefinitions() {
    return [
      'opens' => $this->t('Email Opens'),
      'processed' => $this->t('Processed Emails'),
      'requests' => $this->t('Email Requests'),
      'clicks' => $this->t('Email Clicks'),
      'delivered' => $this->t('Delivered Emails'),
      'deferred' => $this->t('Deferred Emails'),
      'unsubscribes' => $this->t('Unsubscribes'),
      'unsubscribe_drops' => $this->t('Dropped Due to Unsubscribe'),
      'invalid_emails' => $this->t('Invalid Emails'),
      'bounces' => $this->t('Bounced Emails'),
      'bounce_drops' => $this->t('Dropped Due to Bounce'),
      'unique_clicks' => $this->t('Unique Clicks'),
      'blocks' => $this->t('Blocked Emails'),
      'spam_report_drops' => $this->t('Dropped Due to Spam Reports'),
      'spam_reports' => $this->t('Spam Reports'),
      'unique_opens' => $this->t('Unique Opens'),
    ];
  }

}
